# TP5AsplinesParamStudents.py

"""=======================================================
Parametric cubic SPLINE interpolation :
  interpolation of data acquired with the mouse
  with 
    uniform parameterization
    Chebyshev parameterization
    chordal parameterization
Then, comparison with polynomial interpolation
=========================================================="""
import numpy as np
import matplotlib.pyplot as plt
from TP5Asplines import splineC2NatNU

# Mouse acquisition of a polygon : right click to stop
# the returned values x, y are lists
def PolygonAcquisition(color1,color2) :
    x = []  # x is an empty list
    y = []
    coord = 0
    while coord != []:
        coord = plt.ginput(1, mouse_add=1, mouse_stop=3, mouse_pop=2)
        # coord is a list of tuples : coord = [(x,y)]
        if coord != []:
            xx = coord[0][0]
            yy = coord[0][1]
            plt.plot(xx,yy,color1,markersize=8)
            x.append(xx)
            y.append(yy)
            plt.draw()
            if len(x) > 1 :
                plt.plot([x[-2],x[-1]],[y[-2],y[-1]],color2)
    return x,y

"""------------------------------------------------------
MAIN PROGRAM :
------------------------------------------------------"""
plt.cla()
fig = plt.figure(1,(8,8))
ax = fig.add_subplot(111)
xlim = (-10, 10)
ylim = (-10, 10)
ax.set_xlim(xlim)
ax.set_ylim(ylim)

# Acquisition of a polygon 
xi, yi = PolygonAcquisition('oc','c--')
N = np.size(xi) # degree+1

# xi,yi are lists and have to be converted into arrays
xi = np.array(xi)
yi = np.array(yi)

# parameters domain :
a = 0; b = 1

""" --------------
UNIFORM :
------------------"""
# uniform parametrization :
ti = np.linspace(a, b, N) 
# parametric interpolation according the uniform parametrization ti
-----
-----
plt.plot(xs, ys,'b--',lw=2,label='uniform parameterization')



""" --------------
CHEBYSHEV :
------------------"""
# Chebyshev interpolation parameters tch
degree = N - 1
-----
-----
-----
# Chebyshev parametric interpolation
-----
-----
plt.plot(xs, ys,'g',lw=1,label='Chebyshev parameterization')


""" --------------
CHORDAL :
------------------"""
# chordal parameterization :
tc = np.zeros(N)
-----
-----
-----
-----

# parametric interpolation according the chordal parametrization tc
-----
-----
plt.plot(xs, ys,'r--',lw=2,label='chordal parameterization')

plt.legend(loc='best') 


"""===============================================================
COMPARISON with polynomial interpolation :
    ---> nothing to do here
=================================================================="""
from TP3B_Newton import NewtonInterpol
# --> of course the script TP3B_Newton.py 
#     must be in the same folder

nbt = 500
#------------------------
# uniform
# parametric polynomial interpolation according the ti
tx = NewtonInterpol(ti,xi,a,b,nbt)
ty = NewtonInterpol(ti,yi,a,b,nbt)
plt.plot(tx, ty,'c--',lw=1,label='POLYNOMIAL : uniform parameterization')

#------------------------
# Chordal
# parametric polynomial interpolation according the tc
tx = NewtonInterpol(tc,xi,a,b,nbt)
ty = NewtonInterpol(tc,yi,a,b,nbt)
plt.plot(tx, ty,'c:',lw=1,label='POLYNOMIAL : chordal parameterization')

plt.legend(loc='best') 
