# TP5AsplineStudents.py

"""===========================================================
Natural interpolating C2 cubic Spline
    -> U=Uniform : evenly spaced points :
       main function --> (xs,ys) = splineC2NatU(xi,yi)
    -> NU=nonUniform : arbitrary points in increasing order :
       main function --> (xs,ys) = splineC2NatNU(xi,yi)
    -> comparison with the scipy cubic spline 
=============================================================="""
import numpy as np
import matplotlib.pyplot as plt

# Test functions
def f0(x):
    return np.sin(x**2 - 2*x + 1) + (np.cos(-x**3 - x))**2
def f1(x):
    return -1 + 2 * np.sin(2*x) / x
def f2(t):
    return np.cos(-t**2 / 9.0)
    
# Cubic Hermite basis over [0,1] :
def H0(t) :
    y = 1 - 3 * t**2 + 2 * t**3
    return y
def H1(t) :
    y = t - 2 * t**2 + t**3
    return y    
def H2(t) :
    y = - t**2 + t**3
    return y
def H3(t) :
    y = 3 * t**2 - 2 * t**3
    return y   

# Cubic Hermite interpolation over 2 points
def HermiteC1b(x0,y0,y0p,x1,y1,y1p):
    """ Cubic Hermite interpolation of order 1 over 2 points x0 < x1
        (interpolation of value + first derivative)
        Remark : no plotting in this version HermiteC1b()
        Input :
            x0,y0,y0p,x1,y1,y1p = Hermite data of order 1 (real values)
        Output :
            x = sampling of 100 values in interval [x0,x1]
            y = image of x by the cubic Hermite interpolant
    """
    x = np.linspace(x0,x1,100)
    h = x1 - x0
    t = (x - x0) / h
    y = y0 * H0(t) + y0p * h * H1(t) + y1p * h * H2(t) + y1 * H3(t)
    return x, y

# Cubic interpolating Hermite spline C1-C1 of n points
def splineC1C1(xi,yi,yip):
    """ Hermite C1 spline interpolating 'C1 data' (xi,yi,yip)
        --> the data are C1 (Hermite data of order one)
        --> the constructed spline is C1
        Input :
            xi,yi,yip = 3 arrays of size N (points, values, derivatives)
        Output :
            xs = sampling of 100*(N-1) values in [x_first,x_last]
            ys = image of xs by the the Hermite C1 spline
    """
    N = np.size(xi)
    xs = []
    ys = []
    for i in range(N-1):
        -----
        -----
        -----
    return xs, ys


def splineC2NatU(xi,yi):
    """ Natural C2 cubic spline interpolating data (xi,yi)
        U=uniform -> values xi are evenly spaced, in increasing order
        Input:
            xi,yi = vectors of data to be interpolated
        Output :
            xs = sampling of [xi_first, xi_last]
            ys = image of xs by the spline
    """
    n = np.size(xi)
    h = (xi[n-1] - xi[0]) / (n-1)
    # construction of the spline matrix A :
    -----
    -----
    -----
    -----
    # construction of the second member b :
    b = np.zeros(n)
    -----
    -----
    -----
    -----
    # determination of derivatives yip at points xi (for C2 contact) :
    -----
    # evaluation of the spline :
    -----
    return (xs, ys)
    
def splineC2NatNU(xi,yi):
    """ Natural C2 cubic spline interpolating data (xi,yi)
        NU=nonUniform -> arbitrary values xi in increasing order
        Input:
            xi,yi = vectors of data to be interpolated
        Output :
            xs = sampling of [xi_first, xi_last]
            ys = image of xs by the spline
    """
    n = np.size(xi)
    h = xi[1:n] - xi[0:n-1]
    # construction of the spline matrix :
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    # construction of the second member :
    b = np.zeros(n)
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    # determination of derivatives yip at points xi :
    -----
    # evaluation of the spline :
    -----
    return (xs, ys)
      
"""------------------------------------------------------
MAIN PROGRAM :  
------------------------------------------------------"""
plt.clf()
# The initial test curve :    
a = -1 ; b = 2 # for function f0
#a = 0 ; b = 10 # for function f2
t = np.linspace(a,b,500)
y = f0(t) 
plt.plot(t,y,'c:',lw=1,label='initial function')    

# UNIFORM data to be interpolated :
N = 12
xi = np.linspace(a,b,N)
yi = f0(xi)
plt.plot(xi,yi,'co',label='uniform data')

#---------------------------------------------------------
# UNIFORM natural cubic spline :
(xs, ys) = splineC2NatU(xi,yi)
plt.plot(xs,ys,'b--',label='natural spline U')

#---------------------------------------------------------
# NON UNIFORM natural cubic spline : (CASE 1)
# --> the data are uniform ==> we must get the same spline as above !!
(xs, ys) = splineC2NatNU(xi,yi)
plt.plot(xs,ys,'b',label='natural spline NU, uniform data')

#---------------------------------------------------------
# NON UNIFORM natural cubic spline : (CASE 2)
# --> we generate NON uniform data with np.random.rand()
#    (random samples from a uniform distribution over [0,1[)
xiNU = a + (b-a) * np.random.rand(N)
xiNU = np.sort(xiNU)
yiNU = f0(xiNU)
plt.plot(xiNU,yiNU,'yo',label='NU data')
(xs, ys) = splineC2NatNU(xiNU,yiNU)
plt.plot(xs,ys,'y',label='natural spline NU, NU data')

plt.legend(loc='best')
