# TP7-LSAwithConstraintsPart2Students.py

"""
Least Squares Approximation with Integral Constraint
Different steps :
- first (and optional): classical least squares approximation (LSA)
    (without constraints)
- then LSA with 2 constraints :
    1) integral constraint
    2) we impose the approximant to go through point x0,y0
"""
#####################################################
## FUNCTIONS
#####################################################
import numpy as np
import matplotlib.pyplot as plt

def LeastSquares(xi,yi,degree):
    """ Determination of the best polynomial of degree "degree"
        approximating data (xi,yi) according the least squares method
        Returns :
            cf = the monomial coefficients of this polynomial
            res = the residual standard deviation
    """
    nblin = np.size(xi)
    nbcol = degree + 1
    # matrix A (monomial basis)
    A = np.ones((nblin,nbcol))
    for k in range(1,nbcol):
        -----
    # normal equations and solution:
    -----
    -----
    -----
    # Residual standard deviation:
    p = cf[degree]
    for k in range(degree-1, -1, -1):
        p = cf[k] + p * xi
    resError = sum( (yi - p)**2 )
    res = np.sqrt( resError / np.size(xi) )
    return cf, res

def SimpsonMethod(f, a, b, n):
    """ Simpson method for determining an approximation of 
        the integral of the function f over the interval [a,b]
        We consider notations of the course : h = t_{k+1} - t_k
        Input : f = function
              : a,b = bounds of the interval
              : n = integer (N=2^n is the number of intervals)
        Output : S = Simpson approximation of the integral
    """
    N = 2**n
    h = (b-a) / N
    # partial sum S1
    -----
    -----
    -----
    # partial sum S2
    -----
    -----
    -----
    # sum S
    S = (f(a) + f(b)) / 6. + S1 + S2
    S *= h
    return S


# functions to be approximated :
def g1(x):
    return 1 + x + 2 * np.sin(x) + np.cos(5*x) 
def g2(x):
    return np.sin(x**2 - 2*x + 1) + (np.cos(x**3 + x))**2
def g3(x):
    return np.abs(x)

# Approximating functions vj(t) = x^j, j = 0,1,...,n
#def vj(j,x):
#    return x**j

# Approximating Functions in a list :
def f0(x):
    return np.ones(np.size(x))
def f1(x):
    return x
def f2(x):
    return x**2
def f3(x):
    return x**3
def f4(x):
    return x**4
def f5(x):
    return x**5

def ff0(x):
    return np.ones(np.size(x))
def ff1(x):
    return np.cos(2*x)
def ff2(x):
    return np.sin(2*x)
def ff3(x):
    return np.cos(3*x)
def ff4(x):
    return np.sin(3*x)

tf = [f0,f1,f2,f3,f4,f5]
#tf = [ff0,ff1,ff2,ff3,ff4]


#####################################################
## MAIN
#####################################################
plt.cla()
plt.grid()
# Plot of the function to be approximated :
f = g1; a =  0; b = 2*np.pi;
#f = g2; a = -1; b = 2;
#f = g3; a = -1; b = 1;
n = 8 # number of evaluation points is 2^n
t = np.linspace(a,b,2**n) # for graph plotting
plt.plot(t,f(t))

# Points to be approximated
N = 9
ti = np.linspace(a,b,N)
plt.plot(ti,f(ti),'ob')

###################################
# 1) classical polynomial 
#    Least Squares Approximation
###################################
degree = len(tf) - 1
cf, res = LeastSquares(ti,f(ti),degree)
# Horner evaluation and plotting
p = cf[degree]
for k in range(degree-1, -1, -1):
    -----
plt.plot(t,p,'--r',lw=1) 

###################################
# 2) LSA with integral constraint
###################################
# Integral of the function f 
# (optional : we can fix any value for the integral of the approximation) :
If = SimpsonMethod(f, a, b, n) # 2^n evaluation points

# determination of vectors Vj (on the rows) :
nbf = len(tf) # number of approximating functions
Vj = np.zeros((nbf,N))
for i in range(nbf):
    -----

# for second member
Y = -----

# integral values of approximating functions
aj = np.zeros(nbf)
for j in range(nbf):
    -----

# We now add a constraint :
# we impose the approximant to go through point x0,y0
x0 = 1.
y0 = 2.
plt.plot(x0,y0,'sg')
cj = np.zeros(nbf)
for i in range(nbf):
    -----

# Main matrix
A = np.zeros((nbf+2,nbf+2))
# First (n+1,n+1) square of the main matrix
for i in range(nbf):
    for j in range(nbf):
        -----
-----
-----
-----
-----

# Second member bb :
bb = np.zeros(nbf+2)
for i in range(nbf):
    -----
-----
#bb[nbf] = 0.8
-----

# Resolution
x = np.linalg.solve(A,bb)
# and evaluation :
u = np.zeros(np.size(t))
for i in range(nbf):
    -----
plt.plot(t,u)

