# TP9ASubdivisionStudents.py
"""----------------------------------------------------
Subdivision of a Bézier curve according parameter 1/2
-------------------------------------------------------"""
import numpy as np
import matplotlib.pyplot as plt
from scipy.special import binom

def Bernstein(n, k, x):
    coeff = binom(n, k)
    return coeff * x**k * (1 - x)**(n - k)

def BezierPlot(ax,xptcontrole,yptcontrole,color3):
    degree = len(xptcontrole) - 1
    nbti = 200
    ttrace = np.linspace(0,1,nbti)
    tx = np.zeros(nbti)
    ty = np.zeros(nbti)
    for k in range(degree+1):
        poly = Bernstein(degree,k,ttrace)
        tx = (tx + xptcontrole[k]*poly)
        ty = (ty + yptcontrole[k]*poly)
    ax.plot(tx,ty,color3,lw=1)
    plt.draw()

def PolygonAcquisition(ax,color1,color2):
    """ Acquisition of a 2D polygon in the window with subplot "ax" 
        right click stop the acquisition
    """
    x = []  # x is an empty list
    y = []
    coord = 0
    while coord != []:
        coord = plt.ginput(1, mouse_add=1, mouse_stop=3, mouse_pop=2)
        # coord is a list of tuples : coord = [(x,y)]
        if coord != []:
            xx = coord[0][0]
            yy = coord[0][1]
            ax.plot(xx,yy,color1,ms=8)
            x.append(xx)
            y.append(yy)
            plt.draw()
            if len(x) > 1 :
                ax.plot([x[-2],x[-1]],[y[-2],y[-1]],color2)
    return x,y

def OneSubdivBezierCurve(xpt, ypt):
    """ One subdivision of the input control polygon according parameter 1/2
        Return the two subdivided control polygons : 
            xpt1, ypt1
            xpt2, ypt2
    """
    n = len(xpt) - 1 # degree of the Bézier curve
    xp = np.zeros(2*n+1)
    yp = np.zeros(2*n+1)
    # Initialization
    for i in range(n+1):
        xp[2*i] = xpt[i]
        yp[2*i] = ypt[i]
    # Main loop
    for j in range(1,n+1):
        for i in range(n+1-j):
            k = -----
            xp[k] = -----
            yp[k] = -----
    # We split the subivided polygon in two control polygons
    xpt1 = -----
    xpt2 = -----
    ypt1 = -----
    ypt2 = -----  
    return xpt1, ypt1, xpt2, ypt2          


############################################
if __name__ == '__main__':
   
    plt.close()
    minmax = 5
    fig = plt.figure(figsize=(12,8))
    ax = fig.add_subplot(111)
    ax.set_xlim((-minmax,minmax))
    ax.set_ylim((-minmax,minmax))
    ax.set_xlabel('x-axis')
    ax.set_ylabel('y-axis')
    ax.set_title("Acquisition window") 
    
    # Bézier curve acquisition (of its control polygon)
    xp0, yp0 = PolygonAcquisition(ax,'ob','b')
    # and display of of the associated Bézier curve thanks to 
    # the Bernstein parameterization
    BezierPlot(ax,xp0,yp0,'b:')
    
    # Number of subdivision steps according parameter t=1/2
    N = 3
    # degree of the Bézier curve :
    n = len(xp0) - 1 
    # we now apply N subdivision steps :    
    xpS = xp0
    ypS = yp0
    for k in range(N):
        xpSnew = np.array([])
        ypSnew = np.array([])
        for i in range(2**k):
            -----
            -----
            -----
            -----
            -----
            -----
            -----
            -----
        xpS = xpSnew
        ypS = ypSnew
        plt.plot(xpS,ypS)
        plt.draw()
        print("step number is :",k)
        plt.waitforbuttonpress()
        